#!/bin/sh

if [ -z "${PKGROOT}" ]; then
    export PKGROOT=$(realpath ../)
fi

export SSLROOT=${PKGROOT}/ssl
export PATH=${PKGROOT}/bin:${PATH}
export LD_LIBRARY_PATH=${PKGROOT}/lib

CONFIG=${PKGROOT}/config.json
SERVICE=${PKGROOT}/bin/service.sh

if [ ! -f ${CONFIG} ]; then
    ${PKGROOT}/package.sh default 1>&2
fi

LOGPATH=/mnt/ramdisk/pair.log
PCODE_LCK=/mnt/ramdisk/pcode.lck
REQUIRED_KEYS='["ioTEndpoint", "deviceID", "signalServer", "userKey", "ioTCredentialEndpoint", "private", "cert"]'

TEMP_CONFIG_VAL=$(mktemp -p ${PKGROOT} param-XXXXXX)
jq -r '[.vivocloud.enable, .vivocloud.apiserver, .iotbus.endpoint, .iotbus.clientid, .webrtc.deviceID] | map(tostring) | join("|")' ${CONFIG} > ${TEMP_CONFIG_VAL}
IFS="|" read SERVICE_ENABLE API_SERVER IOT_ENDPOINT CLIENT_ID DEVICE_ID < ${TEMP_CONFIG_VAL}
rm -f ${TEMP_CONFIG_VAL}

URL_PAIRDEVICE=${API_SERVER}"/login.html#/login"

wait_process()
{
	while IFS= read -r pid
	do
		while kill -0 ${pid} &> /dev/null; do sleep 0.1; done
	done < "$1"
}

build_pair()
{
	TEMPPAIR=$(mktemp -p ${PKGROOT} pairgen-XXXXXX)
	pairgen -B > ${TEMPPAIR}
	echo "$(date) build pair" >> ${LOGPATH}
	cat ${TEMPPAIR} >> ${LOGPATH}
	#suspend service during pairing
	${SERVICE} suspend_for_pair
}

concat_pair()
{
	PCODE=$(jq -r '.pcode' ${TEMPPAIR})
	PCODE_ENDPOINT="${URL_PAIRDEVICE}?pcode=${PCODE}"
	if [ -z ${PCODE} ]; then
		PCODE_STATUS="fail"
		PCODE_ENDPOINT=""
	else
		PCODE_STATUS="pairing"
	fi

	cat <<- EOF | jq -s '.[0] * .[1]' - ${TEMPPAIR} > ${PCODE_LCK}
	{
		"status":"${PCODE_STATUS}",
		"IoTEndpoint": "${IOT_ENDPOINT}",
		"ClientID": "${CLIENT_ID}",
		"DeviceID": "${DEVICE_ID}",
		"endpoint":"${PCODE_ENDPOINT}",
		"expiration": $(($(date +%s)+600))
	}
	EOF
	#print pcode and TS as send_json input
	cat ${TEMPPAIR}
	rm -f ${TEMPPAIR}
}

check_start()
{
	TEMP_PCODE_VAL=$(mktemp -p ${PKGROOT} param-XXXXXX)
	jq -r '[.pcode, .expiration] | map(tostring) | join("|")' ${PCODE_LCK} > ${TEMP_PCODE_VAL}
	IFS="|" read PCODE EXPIRATION < ${TEMP_PCODE_VAL}
	rm -f ${TEMP_PCODE_VAL}

	while [ -f ${PCODE_LCK} ]; do
		CUR_PCODE=$(jq -r '.pcode' ${PCODE_LCK})
		if [ -z ${CUR_PCODE} ]; then
			rm -f ${PCODE_LCK}
			exit 0
		fi

		if [ ${SERVICE_ENABLE} = "false" ]; then
			rm -f ${PCODE_LCK}
			exit 0
		fi

		if [ ${CUR_PCODE} != "${PCODE}" ]; then
			exit 0
		fi

		CUR_TIME=$(date +%s)
		if [ "${CUR_TIME}" -ge "${EXPIRATION}" ]; then
			rm -f ${PCODE_LCK}
			TIMEOUT=true
			break
		fi
		sleep 5
	done
	[ "${TIMEOUT}" == true ] && ${SERVICE} resume_for_pair
}

check_status()
{
	if [ "${1}" == 200 ]; then
		keyring im ${SSLROOT}/awsiot/IoTPrivateKey.pem &> /dev/null &
		${PKGROOT}/bin/initconf.sh &> /dev/null &
		${PKGROOT}/package.sh start &> /dev/null &
	fi
}

save_json()
{
	if [ ! -f ${PCODE_LCK} ]; then
		echo {} | jq '.status = 400 | .message = "Pcode expired."'
		return
	fi
	REQ_DATA=$(jq '.' -)
	echo "$(date) save info" >> ${LOGPATH}
	echo "${REQ_DATA}" | jq 'del(.private, .cert)' >> ${LOGPATH}

	VALID=$(echo "${REQ_DATA}" | jq --argjson PARAM "$REQUIRED_KEYS" 'keys | contains($PARAM)')
	if [ "${VALID}" != true ]; then
		echo {} | jq '.status = 500 | .message = "Invalid parameters."'
		return
	fi

	TRIM_DATA=$(echo "${REQ_DATA}" | sed 's/: "[^"]*"/\n&/g;:a;s/\(\n: "[^"]*\) /\1/g;ta;s/\n//g')
	VALID=$(echo "${TRIM_DATA}" | jq 'to_entries | map(select(.key == ("ioTEndpoint", "deviceID", "signalServer", "userKey", "ioTCredentialEndpoint", "private", "cert"))) | all(.value != null and .value != '\"\"')')
	if [ "${VALID}" != true ]; then
		echo {} | jq '.status = 500 | .message = "Invalid parameters."'
		return
	fi

	# Add this judgement when next app version release.
	# if [ $(echo "${REQ_DATA}" | jq -r '.value') != $(jq -r '.pcode' ${PCODE_LCK}) ]; then
	# 	echo {} | jq '.status = 500 | .message = "Invalid parameters."'
	# 	return
	# fi

	rm -f ${PCODE_LCK}

	# start blackbird early
	TEMP_BLACKBIRD_VAL=$(mktemp -p ${PKGROOT} param-XXXXXX)
	echo ${REQ_DATA} | jq -r '[.signalServer, .deviceID] | map(tostring) | join("|")'> ${TEMP_BLACKBIRD_VAL}
	IFS="|" read ENDPOINT DEVICE < ${TEMP_BLACKBIRD_VAL}
	rm -f ${TEMP_BLACKBIRD_VAL}
	export ENDPOINT DEVICE
	${PKGROOT}/init.d/blackbird start &

	# import cert / private key
	mkdir -p ${SSLROOT}/awsiot
	keyring rm ${SSLROOT}/awsiot/IoTPrivateKey.pem &> /dev/null
	echo "${REQ_DATA}" | jq -r '.cert' > ${SSLROOT}/awsiot/IoTCertificate.crt
	echo "${REQ_DATA}" | jq -r '.private' > ${SSLROOT}/awsiot/IoTPrivateKey.pem

	# update config.json
	TEMPCONF=$(mktemp -p ${PKGROOT} config-XXXXXX)
	echo "${REQ_DATA}" | jq '{iotbus: {clientid: .deviceID, endpoint: .ioTEndpoint}, webrtc: {deviceID: .deviceID, endpoint: .signalServer}, cloudrecd: {enable: .cloudRecordingEnable, cameraID: .deviceID, identityID: .userKey, ioTCredentialEndpoint: .ioTCredentialEndpoint, s3Bucket: .bucket}}' > ${TEMPCONF}
	jq -s '.[0] * .[1]' ${CONFIG} ${TEMPCONF} > ${CONFIG}~
	rm -f ${TEMPCONF}
	mv ${CONFIG}~ ${CONFIG}

	# update AWS shadow
	echo "$(date) mosquitto publish" >> ${LOGPATH}
	TEMPERROR=$(mktemp -p ${PKGROOT} error-XXXXXX)
	TEMPPIDS=$(mktemp -p ${PKGROOT} pids-XXXXXX)
	${PKGROOT}/init.d/pushcap start 2>> ${TEMPERROR} 1>> ${LOGPATH} &
	echo "$!" >> ${TEMPPIDS}

	wait_process ${TEMPPIDS}
	rm -f ${TEMPPIDS}

	if [ -s ${TEMPERROR} ]; then
		${PKGROOT}/init.d/blackbird stop
		echo {} | jq '.status = 600 | .message = "Publish MQTT failed."'
	else
		echo {} | jq '.status = 200 | .message = "Done."'
	fi

	rm -f ${TEMPERROR}
}

send_json()
{
	printf "Content-type: application/json; charset=utf-8\r\n"
	printf "\r\n"
	jq -c '.' ${1}
	printf "\r\n"
}

trap "exit 0" INT TERM

case ${REQUEST_METHOD} in
	GET)
		build_pair
		concat_pair | send_json -
		#close stdout/stderr
		exec 1>&-
		exec 2>&-
		# open stdout/stderr
		exec 1>&0
		exec 2>&0
		check_start
		;;
	PUT)
		RES=$(save_json)
		echo "${RES}" | send_json -
		STATUS=$(echo "${RES}" | jq -r '.status')
		check_status $STATUS
		;;
	*)
		echo "Status: 405 Method Not Allowed"
		echo {} | jq  '.message=$message' --arg message "ERROR: Unknown METHOD. ${REQUEST_METHOD}" | send_json -
		exit 1
		;;
esac

exit 0
